/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package jdct;

import jdct.*;
import dbStatic.* ;
import java.lang.* ;
import java.awt.*;
import java.awt.event.*;

class JdctLinkFieldInstance extends Panel 
implements ActionListener, FocusListener
{
	private JdctFrame frame;
	private JdctInstanceGui recordGui;
	private String recordName;
	private String fieldName;
	private int linkType;

	private JdctTextFieldInstance linkFieldText;
	private Button formButton;
	private FormEntryDialog formEntryDialog = null;


	JdctLinkFieldInstance(JdctFrame frame,
	    JdctInstanceGui recordGui, String recordName, String fieldName,
	    String text,int linkType)
	    {

		super(new BorderLayout(0,0));

		this.frame = frame;
		this.recordGui = recordGui;
		this.recordName = recordName;
		this.fieldName = fieldName;
		this.linkType = linkType;

		linkFieldText = new JdctTextFieldInstance(frame,
		    recordGui, recordName,fieldName,text);
		linkFieldText.addFocusListener(this);
		formButton = new Button("Form");
		formButton.addActionListener(this);

		this.add("East",formButton);
		this.add("Center",linkFieldText);

	}

	public void actionPerformed(ActionEvent e)
	    {
		String error = null;

		String action = e.getActionCommand();

		// location for error message dialog
		    Component c = (Component)e.getSource();
		Point p = c.getLocationOnScreen();
		Dimension d = c.getSize();
		//Point location = new Point(p.x - 200, p.y + d.height);
		Point location =   new Point(p.x - 200, p.y );

		DbBase dbBase = frame.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if(!dbEntry.findRecord(recordName)) {
			recordGui.messageDialog("ERROR",
			    "Record " + recordName + " Not Found?",location);
			dbEntry.free();
			return;
		}
		if(!dbEntry.findField(fieldName)) {
			recordGui.messageDialog("ERROR",
			    "Field " + fieldName + " Not Found?",location);
			dbEntry.free();
			return;
		}

		// Form button pressed
		    if (action == "Form") {
			int  numFormLines = dbEntry.allocForm();
			if(numFormLines == 0) {
				recordGui.messageDialog(" ","No associated form",location);
				dbEntry.free();
				return;
			}

			String[]  promptList = dbEntry.getFormPrompt();
			String[]  fieldValueList = dbEntry.getFormValue();
			linkType = dbEntry.getLinkType();
			if (formEntryDialog == null) {
				formEntryDialog = new FormEntryDialog(recordGui,fieldName,numFormLines,
				    promptList,fieldValueList,linkType,location);
			} else {
				formEntryDialog.toFront();
			}
			dbEntry.free();
			return;
		}

		dbEntry.free();
	}

	public void removeDialog()
	    {
		if (formEntryDialog != null) {
			formEntryDialog.dispose();
		}
	}
	public void focusGained(FocusEvent e)
	    {
	}
	public void focusLost(FocusEvent e)
	    {
		// JdctTextFieldInstance enter pressed
		    if (e.getSource() instanceof JdctTextFieldInstance) {

			update();
			return;
		}
	}

	public void update()
	    {
		// location for error message dialog
		    Point p = linkFieldText.getLocationOnScreen();
		Point location = new Point(p.x - 200, p.y );

		DbBase dbBase = frame.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if(!dbEntry.findRecord(recordName)) {
			recordGui.messageDialog("ERROR",
			    "Record " + recordName + " Not Found?",location);
			dbEntry.free();
			return;
		}
		if(!dbEntry.findField(fieldName)) {
			recordGui.messageDialog("ERROR",
			    "Field " + fieldName + " Not Found?",location);
			dbEntry.free();
			return;
		}
		linkFieldText.setText(dbEntry.getString());

		if (formEntryDialog != null) {
			int numFormLines = dbEntry.allocForm();
			if(numFormLines == 0) {
				recordGui.messageDialog("No associated form",
				    "  Most likely missing device definition ", location);
				dbEntry.free();
				return;
			}

			String[] promptList = dbEntry.getFormPrompt();
			String[] fieldValueList = dbEntry.getFormValue();
			formEntryDialog.update(numFormLines,
			    promptList,fieldValueList,dbEntry.getLinkType());
			//formEntryDialog.toFront();
			dbEntry.free();
		}
	}

	class FormEntryDialog extends Dialog implements ActionListener,WindowListener
	    {
		private final int FORM_UPDATE_LENGTH = 30;
		private TextField[] textField = null;
		private Button convertButton = null;
		Panel entryPanel = null;
		Panel panel = null;

		FormEntryDialog(Frame frame,String fieldName, int numFormLines,
		    String[] promptList,String[] fieldValueList,int linkType,
		    Point location)
		    {
			super(frame,fieldName,false);
			addWindowListener(this);

			entryPanel = new Panel();
			entryPanel.setLayout(new GridLayout(0,1));
			textField = new TextField[numFormLines];

			for (int i=0;i<numFormLines;i++) {
				Panel panel = new Panel();
				panel.setLayout(new BorderLayout(0,0));
				panel.add("West",new Label(promptList[i]));
				textField[i] = new TextField();
				textField[i].setText(fieldValueList[i]);
				textField[i].setColumns(FORM_UPDATE_LENGTH);
				panel.add("East",textField[i]);
				entryPanel.add(panel);
			}
			this.add("North", entryPanel);

			panel = new Panel();
			panel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
			Button okButton = new Button("Ok");
			okButton.addActionListener(this);
			panel.add(okButton);
			Button cancelButton = new Button("Cancel");
			cancelButton.addActionListener(this);
			panel.add(cancelButton);
			convertButton = new Button("Constant");
			if ( DbEntry.DCT_LINK_CONSTANT == linkType){
				convertButton.setLabel("PV link");
				panel.add(convertButton);
			} else if ( DbEntry.DCT_LINK_PV == linkType){
				convertButton.setLabel("Constant");
				panel.add(convertButton);
			}
			convertButton.addActionListener(this);
			this.add("South", panel);

			this.pack();
			this.setLocation(location);
			this.setVisible(true);
		}

		public void windowClosing(WindowEvent e)
		    {
			dispose();
			formButton.transferFocus();
		}
		public void windowOpened(WindowEvent e) {
		}
		public void windowClosed(WindowEvent e) {
		}
		public void windowIconified(WindowEvent e) {
		}
		public void windowDeiconified(WindowEvent e) {
		}
		public void windowActivated(WindowEvent e) {
		}
		public void windowDeactivated(WindowEvent e) {
		}

		public void dispose()
		    {
			JdctLinkFieldInstance.this.formEntryDialog = null;
			super.dispose();
		}

		public void actionPerformed(ActionEvent e)
		    {
			String error = null;

			String action = e.getActionCommand();
			if (action == "Cancel") {
				dispose();
				formButton.transferFocus();
				return;
			}

			// location for error message dialog
			    Component c = (Component)e.getSource();
			Point p = c.getLocationOnScreen();
			Dimension d = c.getSize();
			Point location =   new Point(p.x - 200, p.y );

			DbBase dbBase = frame.getDbBase();
			DbEntry dbEntry = new DbEntry(dbBase);
			if(!dbEntry.findRecord(recordName)) {
				recordGui.messageDialog("ERROR",
				    "Record " + recordName + " Not Found?",location);
				dbEntry.free();
				return;
			}
			if(!dbEntry.findField(fieldName)) {
				recordGui.messageDialog("ERROR",
				    "Field " + fieldName + " Not Found?",location);
				dbEntry.free();
				return;
			}

			// Convert button pressed
			    if (action == "Constant") {
				dbEntry.cvtLinkToConstant();
				JdctLinkFieldInstance.this.update();
				return;
			}
			if (action == "PV link") {
				dbEntry.cvtLinkToPvlink();
				JdctLinkFieldInstance.this.update();
				return;
			}

			// FormEntryDialog OK button pressed
			    if (action == "Ok") {
				int  numFormLines = dbEntry.allocForm();
				String[]  errorList = new String[numFormLines];

				// get textField values
				    String[] userValueList = new String[numFormLines];
				for (int i=0;i<numFormLines;i++) {
					userValueList[i] = textField[i].getText();
				}

				// get database values
				    String[] fieldValueList = null;
				fieldValueList = dbEntry.getFormValue();

				// save database values
				    String[]  lastGoodList = new String[numFormLines];
				for(int i=0; i<numFormLines; i++) {
					lastGoodList[i] = fieldValueList[i];
				}

				// if illegal data entered on any form line 
				    if ( dbEntry.verifyForm(userValueList,errorList)) {
					fieldValueList = null;
					fieldValueList = dbEntry.getFormValue();
					String[]  promptList = dbEntry.getFormPrompt();
					TextField[] textFieldList = textField;
					for(int i=0; i<numFormLines; i++) {
						if ( errorList[i] != null && errorList[i].length() > 0) {
							textFieldList[i].selectAll();
							recordGui.messageDialog("ERROR",
							    promptList[i]+" "+userValueList[i]+"  "+errorList[i],location);
							// textFieldList[i].setText(fieldValueList[i]);
							textFieldList[i].setText(lastGoodList[i]);
							textFieldList[i].selectAll();
							textFieldList[i].requestFocus();
							break;
						}
					}
				} else {
					dbEntry.putForm(userValueList);
					linkFieldText.setText(dbEntry.getString());
					formButton.transferFocus();
					dispose();
				}
			}

			dbEntry.free();
		}

		public void update(int numFormLines, String[] promptList,
		    String[] fieldValueList,int linkType)
		    {
			if (entryPanel != null) this.remove(entryPanel);
			entryPanel = new Panel();
			entryPanel.setLayout(new GridLayout(0,1));
			textField = new TextField[numFormLines];

			for (int i=0;i<numFormLines;i++) {
				Panel panel = new Panel();
				panel.setLayout(new BorderLayout(0,0));
				panel.add("West",new Label(promptList[i]));
				textField[i] = new TextField();
				textField[i].setText(fieldValueList[i]);
				textField[i].setColumns(FORM_UPDATE_LENGTH);
				panel.add("East",textField[i]);
				entryPanel.add(panel);
			}
			this.add("North", entryPanel);

			if ( DbEntry.DCT_LINK_CONSTANT == linkType){
				convertButton.setLabel("PV link");
				panel.add(convertButton);
			} else if ( DbEntry.DCT_LINK_PV == linkType){
				convertButton.setLabel("Constant");
				panel.add(convertButton);
			} else {
				if (panel.isAncestorOf(convertButton))
					panel.remove(convertButton);
			}
			this.pack();
		}
	}

}

