/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package jdct;

import jdct.*;
import dbStatic.* ;
import java.lang.* ;
import java.awt.*;
import java.awt.event.*;

class JdctTextFieldInstance extends TextField 
implements ActionListener,FocusListener
{
	private JdctFrame frame;
	private JdctInstanceGui recordGui;

	private String recordName;
	private String fieldName;

	JdctTextFieldInstance(JdctFrame frame, JdctInstanceGui recordGui,
	    String recordName, String fieldName,String text)
	    {
		super(text);
		addFocusListener(this);
		addActionListener(this);
		this.frame = frame;
		this.recordGui = recordGui;
		this.recordName = recordName;
		this.fieldName = fieldName;
	}


	public void focusGained(FocusEvent e)
	    {
	}

	public void focusLost(FocusEvent e)
	    {

		TextField src = (TextField)e.getSource();

		if (textFieldAction(src)) {
			src.requestFocus();
		}
	}

	public void actionPerformed(ActionEvent e)
	    {
		TextField src = (TextField)e.getSource();
		src.transferFocus();
	}

	private boolean textFieldAction(TextField textField)
	    {
		String error = null;

		// location for error message dialog
		    Point p = this.getLocationOnScreen();
		Dimension d = this.getSize();
		//Point location = new Point(p.x - 200, p.y + d.height);
		Point location =   new Point(p.x - 200, p.y );

		DbBase dbBase = frame.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		if(!dbEntry.findRecord(recordName)) {
			recordGui.messageDialog("ERROR",
			    "Record " + recordName + " Not Found?",location);
			dbEntry.free();
			return false;
		}
		if(!dbEntry.findField(fieldName)) {
			recordGui.messageDialog("ERROR",
			    "Field " + fieldName + " Not Found?",location);
			dbEntry.free();
			return false;
		}

		String newValue = this.getText();
		error = dbEntry.verify(newValue);
		if(error!=null) {
			this.selectAll();
			recordGui.messageDialog("Input error",
			    newValue + " " +error,location);
			this.setText(dbEntry.getString());
			this.selectAll();
			dbEntry.free();
			return true;
		} else {
			dbEntry.putString(newValue);
			this.setText(dbEntry.getString());
		}
		dbEntry.free();
		return false;
	}
}
